import json
import argparse
import os
from difflib import SequenceMatcher

def inline_diff(a, b):
    matcher = SequenceMatcher(None, a, b)
    def process_tag(tag, i1, i2, j1, j2):
        if tag == 'replace':
            return '{' + matcher.a[i1:i2] + ' -> ' + matcher.b[j1:j2] + '}'
        if tag == 'delete':
            return '{- ' + matcher.a[i1:i2] + '}'
        if tag == 'equal':
            return matcher.a[i1:i2]
        if tag == 'insert':
            return '{+ ' + matcher.b[j1:j2] + '}'
        assert False, "Unknown tag %r"%tag
    return ''.join(process_tag(*t) for t in matcher.get_opcodes())

def diff_objects(obj1, obj2):
  for k in obj1:
    if obj1[k] != obj2[k]:
      # print(type(obj1[k]))
      if type(obj1[k]) == str and type(obj2[k]) == str:
        print(k, ':', inline_diff(obj1[k], obj2[k]))
      else:
        print(k, ':', obj1[k], '->', obj2[k])

def run(file1, file2):
  with open(file1, 'r') as f:
    data1 = json.loads(f.read())

  with open(file2, 'r') as f:
    data2 = json.loads(f.read())

  r1 = data1['revisions'][-1]
  r2 = data2['revisions'][-1]

  print('=== Resources ===')

  n_res = max(len(r1['resources']), len(r2['resources']))
  for i in range(n_res):
    if i >= len(r2['resources']):
      print('===')
      print('Omitted resource')
      print(r1['resources'][i]['name'])
    elif i >= len(r1['resources']):
      print('===')
      print('Extra resource')
      print(r2['resources'][i]['name'])
    elif r1['resources'][i] != r2['resources'][i]:
      res_differ = False
      for k in r1['resources'][i]:
        v1 = r1['resources'][i][k]
        v2 = r2['resources'][i][k]

        if k != 'id':
          if v1 != v2:
            if not res_differ:
              print('===')
              print('Resources differ:', i)
              res_differ = True
              
            print(k, 'expected:', v1, '!=', 'actual:', v2)

  print('=== Tasks ===')

  if len(r1['tasks']) != len(r2['tasks']):
    raise Exception('Tasks differ')

  n_tasks = len(r1['tasks'])
  for i in range(n_tasks):
    tasks_differ = False
    for k in r1['tasks'][i]:
      v1 = r1['tasks'][i][k]
      v2 = r2['tasks'][i][k]
      
      if k != 'hazards' and k != 'id' and k != 'resources':
        if v1 != v2:
          if not tasks_differ:
            print('===')
            print('Tasks differ:', i)
            tasks_differ = True
            
          print(k, 'expected:', v1, '!=', 'actual:', v2)
          
  print('=== Hazards ===')
  print('Index' + ';' +
        'Type' + ';' +
        'Correct' + ';' +
        'Expected' + ';' + 'Actual' + ';' + 'Description')

  n_shouldmodifycorrect = 0
  n_shouldmodifymodified = 0
  n_shouldmodifytotal = 0

  n_shouldnotcorrect = 0
  n_shouldnotmodified = 0
  n_shouldnottotal = 0
  
  for i in range(n_tasks):
    t1 = r1['tasks'][i]
    t2 = r2['tasks'][i]

    has_gripper = len([x for x in t1['resources'] if x['refId'] == 'GRIPPER']) > 0
    has_motor = len([x for x in t1['resources'] if x['refId'] == 'MOTOR']) > 0

    if has_gripper and has_motor:
      ttype = 'BOTH'
    elif has_gripper:
      ttype = 'GRIPPER'
    elif has_motor:
      ttype = 'MOTOR'
    else:
      ttype = 'NONE'

    tasks_differ = False

    v1 = t1['hazards']
    v2 = t2['hazards']
  
    if len(v1) != len(v2) or len(v1) != 1:
      raise Exception('Hazards modified')
    
    # s1 = v1[0]['mitigation'].replace('Use of a safety-rated proximity sensor to reduce the robot speed when an operator is close to the robot. The smallest possible contact area is ', '').split(' cm²')[0]
    if ttype == 'MOTOR':
      s1 = '1.5'
    elif ttype == 'NONE':
      s1 = '0.75'
    else:
      s1 = '2.0'
    s2 = v2[0]['mitigation'].replace('Use of a safety-rated proximity sensor to reduce the robot speed when an operator is close to the robot. The smallest possible contact area is ', '').split(' cm²')[0]

    correct = float(s1) == float(s2)
    modified = float('0.75') != float(s2) or v2[0]['modification'] is not None
    
    if ttype != 'NONE':
      n_shouldmodifytotal += 1
      if modified:
        n_shouldmodifymodified += 1
      if correct:
        n_shouldmodifycorrect += 1
    else:
      n_shouldnottotal += 1
      if modified:
        n_shouldnotmodified += 1
      if correct:
        n_shouldnotcorrect += 1

    # if v1[0]['modification'] is None and correct:
    if ttype == 'NONE' and correct:
      continue

    print(str(i) + ';' +
      ttype + ';' +
      ('1' if correct else '0') + ';' +
      s1 + ';' + s2 + ';' + t1['description'])
  
  print('correctly changed:', n_shouldmodifycorrect, '/ changed:', n_shouldmodifymodified, '/ should change:', n_shouldmodifytotal,
    ', correct unchanged:', n_shouldnotcorrect, '/ changed:', n_shouldnotmodified, '/ should not change:', n_shouldnottotal,
    'total:', n_tasks)

parser = argparse.ArgumentParser(description='Diff RA component exports')
parser.add_argument('file1', type=str, help='first file')
parser.add_argument('file2', type=str, help='second file')
args = parser.parse_args()

run(args.file1, args.file2)
